//
//  Command.swift
//  CommandDesignPattern
//
//  Created by Gabriel Theodoropoulos.
//  https://serialcoder.dev
//

import Foundation
import SwiftUI

protocol Command: AnyObject {
    var shapeActionReceiver: ShapeActionReceiver { get set }
    func execute()
}


// MARK: - MoveCommand

class MoveCommand: Command {
    var shapeActionReceiver: ShapeActionReceiver
    var direction: MoveDirection
    
    init(shapeActionReceiver: ShapeActionReceiver, direction: MoveDirection) {
        self.shapeActionReceiver = shapeActionReceiver
        self.direction = direction
    }
    
    func execute() {
        shapeActionReceiver.move(direction: direction)
    }
}


// MARK: - PaintCommand

class PaintCommand: Command {
    var shapeActionReceiver: ShapeActionReceiver
    var color: Color
    
    init(shapeActionReceiver: ShapeActionReceiver, color: Color) {
        self.shapeActionReceiver = shapeActionReceiver
        self.color = color
    }
    
    func execute() {
        shapeActionReceiver.paint(color: color)
    }
}
