//
//  CommandDesignPatternApp.swift
//  CommandDesignPattern
//
//  Created by Gabriel Theodoropoulos.
//  https://serialcoder.dev
//

import SwiftUI

@main
struct CommandDesignPatternApp: App {
    var shapeActionReceiver = ShapeActionReceiver()
    var shapeActionInvoker = ShapeActionInvoker()
    
    var body: some Scene {
        WindowGroup {
            ContentView()
                .frame(width: 800, height: 600)
                .environmentObject(shapeActionInvoker)
                .environmentObject(shapeActionReceiver)
        }
        .commands {
            CommandMenu("Move") {
                Button("Up") {
                    shapeActionInvoker.updateShape(using: MoveCommand(shapeActionReceiver: shapeActionReceiver,
                                                                      direction: .up))
                }
                .keyboardShortcut(.upArrow, modifiers: [])
                
                Button("Down") {
                    shapeActionInvoker.updateShape(using: MoveCommand(shapeActionReceiver: shapeActionReceiver,
                                                                      direction: .down))
                }
                .keyboardShortcut(.downArrow, modifiers: [])
                
                Button("Left") {
                    shapeActionInvoker.updateShape(using: MoveCommand(shapeActionReceiver: shapeActionReceiver,
                                                                      direction: .left))
                }
                .keyboardShortcut(.leftArrow, modifiers: [])
                
                Button("Right") {
                    shapeActionInvoker.updateShape(using: MoveCommand(shapeActionReceiver: shapeActionReceiver,
                                                                      direction: .right))
                }
                .keyboardShortcut(.rightArrow, modifiers: [])
            }
            
            CommandMenu("Paint") {
                Button("Blue") {
                    shapeActionInvoker.updateShape(using: PaintCommand(shapeActionReceiver: shapeActionReceiver,
                                                                       color: .blue))
                }
                
                Button("Pink") {
                    shapeActionInvoker.updateShape(using: PaintCommand(shapeActionReceiver: shapeActionReceiver,
                                                                       color: .pink))
                }
                
                Button("Indigo") {
                    shapeActionInvoker.updateShape(using: PaintCommand(shapeActionReceiver: shapeActionReceiver,
                                                                       color: .indigo))
                }
                
                Button("Mint") {
                    shapeActionInvoker.updateShape(using: PaintCommand(shapeActionReceiver: shapeActionReceiver,
                                                                       color: .mint))
                }
            }
        }
    }
}
