//
//  MoveButtons.swift
//  CommandDesignPattern
//
//  Created by Gabriel Theodoropoulos.
//  https://serialcoder.dev
//

import SwiftUI

struct MoveButtonsView: View {
    @EnvironmentObject var shapeActionInvoker: ShapeActionInvoker
    @EnvironmentObject var shapeActionReceiver: ShapeActionReceiver
    
    var body: some View {
        moveButton(title: "Up") {
            shapeActionInvoker.updateShape(using: MoveCommand(shapeActionReceiver: shapeActionReceiver,
                                                              direction: .up))
        }
        moveButton(title: "Down") {
            shapeActionInvoker.updateShape(using: MoveCommand(shapeActionReceiver: shapeActionReceiver,
                                                              direction: .down))
        }
        moveButton(title: "Left") {
            shapeActionInvoker.updateShape(using: MoveCommand(shapeActionReceiver: shapeActionReceiver,
                                                              direction: .left))
        }
        moveButton(title: "Right") {
            shapeActionInvoker.updateShape(using: MoveCommand(shapeActionReceiver: shapeActionReceiver,
                                                              direction: .right))
        }
    }
    
    
    @ViewBuilder
    func moveButton(title: String, action: @escaping () -> Void) -> some View {
        Button {
            action()
        } label: {
            Text(title)
        }
    }
}

struct MoveButtons_Previews: PreviewProvider {
    static var previews: some View {
        MoveButtonsView()
    }
}
